package com.ym.mec.student.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.CloudTeacherOrderDao;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.wrapper.LessonCoursewareWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.student:}/lessonCourseware")
@Api(tags = "课件教材")
@RestController
public class LessonCoursewareController extends BaseController {

	@Resource
	private CoursewareFeignService coursewareFeignService;
	@Resource
	private LessonCoursewareService lessonCoursewareService;
	@Resource
	private SysUserService sysUserService;
	@Resource
	private OrganizationService organizationService;
	@Resource
	private CbsMusicScoreCategoriesService cbsMusicScoreCategoriesService;

    @Resource
    private CloudTeacherOrderDao cloudTeacherOrderDao;

    @Resource
    private MemberRankCourseTypeMapperService memberRankCourseTypeMapperService;

    @Resource
    private MemberRankCategoryMapperService memberRankCategoryMapperService;

    @Resource
    private SysMusicScoreService sysMusicScoreService;

	@ApiOperation(value = "分页查询已添加的课件")
	@PostMapping("/queryLessonCourseware")
	public HttpResponseResult<PageInfo<LessonCoursewareWrapper.LessonCoursewareDto>> queryLessonCourseware(@RequestBody LessonCoursewareWrapper.LessonCoursewareQuery query){
		query.setEnable(true);
		query.setStudentId(sysUserService.getUserId());
		IPage<LessonCoursewareWrapper.LessonCoursewareDto> pages = lessonCoursewareService.selectPage(QueryInfo.getPage(query), query);
        List<LessonCoursewareWrapper.LessonCoursewareDto> records = pages.getRecords();
        if (CollectionUtils.isNotEmpty(records)) {

            // 根据用户会员判断是否锁定
            // 查询有效的会员
            List<Integer> courseIds = getCourseIds();

            // 判断曲目是否在分类中
            for (LessonCoursewareWrapper.LessonCoursewareDto row : records) {
                if (courseIds.contains(row.getId())) {
                    row.setUseStatus("UNLOCK");
                } else  {
                    row.setUseStatus("LOCK");
                }
            }
        }

        return succeed(PageUtil.pageInfo(pages));
	}

	@ApiOperation(value = "获取课件详情")
	@GetMapping("/getLessonCoursewareDetail/{id}")
	public HttpResponseResult<LessonCoursewareWrapper.LessonCoursewareExtend> getLessonCoursewareDetail(@PathVariable("id") Long id){
        CbsLessonCoursewareWrapper.LessonCourseware lessonCourseware = coursewareFeignService.lessonCoursewareDetail(id).feignData();
        if (lessonCourseware == null) {
            return succeed();
        }
        //使用课件ID查询应用这边的ID
        LessonCourseware lessonCourseware1 = lessonCoursewareService.getByLessonCoursewareId(id);
        if (lessonCourseware1 == null) {
            throw new BizException("课件不存在");
        }

        LessonCoursewareWrapper.LessonCoursewareExtend lessonCoursewareExtend = JSONObject
            .parseObject(JSONObject.toJSONString(lessonCourseware), LessonCoursewareWrapper.LessonCoursewareExtend.class);
        // 根据用户会员判断是否锁定
        // 查询有效的会员
        List<Integer> courseIds = getCourseIds();


        // 判断曲目是否在分类中
        if (courseIds.contains(lessonCourseware1.getId())) {
            lessonCoursewareExtend.setUseStatus("UNLOCK");
        } else {
            lessonCoursewareExtend.setUseStatus("LOCK");
        }

        return succeed(lessonCoursewareExtend);
	}

    @NotNull
    private List<Integer> getCourseIds() {
        List<Integer> activationVipIds = cloudTeacherOrderDao.getActivationVipIds(sysUserService.getUserId());

        // 获取会员的曲目分类

        List<MemberRankCourseTypeMapper> courseTypeMapperList = memberRankCourseTypeMapperService.getByMemberRankId(activationVipIds);
        if (CollectionUtils.isEmpty(courseTypeMapperList)) {
            courseTypeMapperList = new ArrayList<>();
        }
        List<Integer> categoryIds = courseTypeMapperList.stream().map(MemberRankCourseTypeMapper::getCourseType)
            .distinct().map(Integer::parseInt).collect(Collectors.toList());
        return categoryIds;
    }

    @ApiOperation(value = "获取课件关联的课程列表")
	@GetMapping("/getLessonCoursewareCourseList/{id}")
	public HttpResponseResult<List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail>> getLessonCoursewareCourseList(@PathVariable("id") Long id,
                                                                                                                          String search,
                                                                                                                          Boolean detailFlag){
        CbsLessonCoursewareWrapper.CoursewareDetailQuery query = new CbsLessonCoursewareWrapper.CoursewareDetailQuery();
        query.setId(id);
        query.setSearch(search);
        query.setDetailFlag(detailFlag);
		List<CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetails = coursewareFeignService.coursewareDetail(query).feignData();
		if (CollectionUtils.isNotEmpty(courseScheduleCoursewareDetails)) {
            SysUser sysUser = sysUserService.getUser();
            // 机构可用曲目分类
            String teachingMaterialIds = organizationService.getTeachingMaterialIds(sysUser.getOrganId().toString(),sysUser.getTenantId());
            List<Integer> categoryIdList = cbsMusicScoreCategoriesService.getAllCategoryIdList(teachingMaterialIds);
            // 学生会员可用曲目分类
            List<Integer> activationVipIds = cloudTeacherOrderDao.getActivationVipIds(sysUser.getId());
            // 获取会员的曲目分类
            List<Integer> categoryIds = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(activationVipIds)) {
                List<MemberRankCategoryMapper> categoryMapperList = memberRankCategoryMapperService.getByMemberRankId(activationVipIds);
                if (CollectionUtils.isEmpty(categoryMapperList)) {
                    categoryMapperList = new ArrayList<>();
                }
                categoryIds = categoryMapperList.stream().map(MemberRankCategoryMapper::getCategoryId)
                        .distinct().collect(Collectors.toList());
            }
            LessonCoursewareWrapper.LessonCoursewareDetailQuery query1 = new LessonCoursewareWrapper.LessonCoursewareDetailQuery();
            query1.setUserType(ClientEnum.STUDENT);
            query1.setTenantCategoriesIdList(categoryIdList);
            query1.setStudentCategoriesIdList(categoryIds);
            //获取课件下面包含的所有的曲目编号
            List<Long> musicScoreIds = lessonCoursewareService.getMusicScoreIds(courseScheduleCoursewareDetails);
            Map<Long, SysMusicScore> musicScoreMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(musicScoreIds)) {
                List<SysMusicScore> musicScores = sysMusicScoreService.getDao().findByCbsId(musicScoreIds,false);
                musicScoreMap = musicScores.stream().collect(Collectors.toMap(e -> Long.parseLong(e.getCbsMusicSheetId()), o -> o));
            }
            for (CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail e : courseScheduleCoursewareDetails) {
                lessonCoursewareService.setTreeResourceId(e.getKnowledgePointList(), query1, musicScoreMap);
            }
            courseScheduleCoursewareDetails.removeIf(courseScheduleCoursewareDetail -> !filterAndUpdateKnowledgePointList(courseScheduleCoursewareDetail));
			String jsonString = JSONObject.toJSONString(courseScheduleCoursewareDetails);
			List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetailList =
                    JSONObject.parseArray(jsonString, LessonCoursewareWrapper.CourseScheduleCoursewareDetail.class);
			return succeed(courseScheduleCoursewareDetailList);
		}
		return succeed(new ArrayList<>());
	}


    private boolean filterAndUpdateKnowledgePointList(CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail courseScheduleCoursewareDetail) {
        List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList = courseScheduleCoursewareDetail.getKnowledgePointList();
        if (CollectionUtils.isEmpty(knowledgePointList)) {
            return false; // 如果 knowledgePointList 为空，直接返回 false，后续会删除该 courseScheduleCoursewareDetail
        }

        // 过滤 KnowledgePointSmall
        knowledgePointList.removeIf(knowledgePointSmall -> !filterKnowledgePoint(knowledgePointSmall));
        courseScheduleCoursewareDetail.setKnowledgePointList(knowledgePointList);

        // 如果过滤后的 knowledgePointList 为空，返回 false
        return CollectionUtils.isNotEmpty(knowledgePointList);
    }

    private boolean filterKnowledgePoint(CbsLessonCoursewareDetailWrapper.KnowledgePointSmall knowledgePointSmall) {
        // 过滤 materialList
//        if (CollectionUtils.isNotEmpty(knowledgePointSmall.getMaterialList())) {
//            filterAndUpdateMaterialList(knowledgePointSmall.getMaterialList(), search);
//        }
        // 过滤子知识点 children
        if (CollectionUtils.isNotEmpty(knowledgePointSmall.getChildren())) {
            knowledgePointSmall.getChildren().removeIf(child -> !filterKnowledgePoint(child));
        }
        // 保留当前的条件：materialList 不为空 或者 children 不为空，**否则删除当前节点**
        return CollectionUtils.isNotEmpty(knowledgePointSmall.getMaterialList()) ||
                CollectionUtils.isNotEmpty(knowledgePointSmall.getChildren());
    }

    private void filterAndUpdateMaterialList(List<CbsLessonCoursewareDetailWrapper.MaterialSmall> materialList, String search) {
        materialList.removeIf(materialSmall -> !materialSmall.getName().contains(search));
    }

	@ApiOperation(value = "获取课程详情")
	@GetMapping("/getLessonCourseDetail/{id}")
	public HttpResponseResult<CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail> getLessonCourseDetail(@PathVariable("id") Long id){
		SysUser sysUser = sysUserService.getUser();
        // 机构可用曲目分类
		String teachingMaterialIds = organizationService.getTeachingMaterialIds(sysUser.getOrganId().toString(),sysUser.getTenantId());
		List<Integer> categoryIdList = cbsMusicScoreCategoriesService.getAllCategoryIdList(teachingMaterialIds);

        // 学生会员可用曲目分类
        List<Integer> activationVipIds = cloudTeacherOrderDao.getActivationVipIds(sysUser.getId());

        // 获取会员的曲目分类

        List<MemberRankCategoryMapper> categoryMapperList = memberRankCategoryMapperService.getByMemberRankId(activationVipIds);
        if (CollectionUtils.isEmpty(categoryMapperList)) {
            categoryMapperList = new ArrayList<>();
        }
        List<Integer> categoryIds = categoryMapperList.stream().map(MemberRankCategoryMapper::getCategoryId)
            .distinct().collect(Collectors.toList());
        LessonCoursewareWrapper.LessonCoursewareDetailQuery query = new LessonCoursewareWrapper.LessonCoursewareDetailQuery();
        query.setLessonCoursewareDetailId(id);
        query.setUserType(ClientEnum.STUDENT);
        query.setTenantCategoriesIdList(categoryIdList);
        query.setStudentCategoriesIdList(categoryIds);
		return succeed(lessonCoursewareService.getLessonCourseDetail(query));
	}

	@ApiOperation(value = "获取课件关联的声部列表")
	@GetMapping("/getLessonCoursewareSubjectList")
	public HttpResponseResult<List<Subject>> getLessonCoursewareSubjectList(){
		return succeed(lessonCoursewareService.getDao().getLessonCoursewareSubjectList());
	}

    @ApiOperation(value = "关联课程类型")
    @PostMapping("/refLevel")
    public HttpResponseResult<List<LessonCoursewareWrapper.RefLevelDetailInfo>> refLevel(@RequestBody LessonCoursewareWrapper.RefLevelQuery query) {
        query.setTeacherId(sysUserService.getUserId());
        query.setClient(ClientEnum.STUDENT);
        return succeed(lessonCoursewareService.refLevel(query));
    }
}
