package com.ym.mec.biz.dal.wrapper;

import com.baomidou.mybatisplus.annotation.TableField;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.dto.im.BasicUserInfo;
import com.ym.mec.biz.dal.enums.ClientEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.models.auth.In;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * 课件教材
 */
@ApiModel(value = "LessonCoursewareWrapper对象", description = "课件教材")
public class LessonCoursewareWrapper {

    @Data
	@Builder
    @AllArgsConstructor
    @NoArgsConstructor
    @ApiModel("课件教材分页查询")
    public static class LessonCoursewareQuery implements QueryInfo {
        private Integer page = 1;

        private Integer rows = 20;

        private String search;

        @ApiModelProperty("声部")
        private Integer subjectId;

        @ApiModelProperty("内容平台课件编号")
        private List<Long> lessonCoursewareIds;

        @ApiModelProperty("启用状态")
        private Boolean enable;

        @ApiModelProperty(value = "学员适用范围ALL,RANGE,DISABLE")
        private String studentRangeType;

        @ApiModelProperty(value = "老师适用范围ALL,RANGE,DISABLE")
        private String teacherRangeType;

        private Integer teacherId;

        private Integer studentId;

        @ApiModelProperty("课程级别")
        private Integer level;
    }

    @Data
	@Builder
    @AllArgsConstructor
    @NoArgsConstructor
    @ApiModel("课件教材dto")
    public static class LessonCoursewareDto{
        private Integer id;

        @ApiModelProperty("课件编号")
		private Long lessonCoursewareId;

        @ApiModelProperty("课件名称")
		private String name;

        @ApiModelProperty("封面")
        private String cover;

        @ApiModelProperty("启用状态")
        private Boolean enable;

        @ApiModelProperty("课时数")
        private Integer courseNum;

        @ApiModelProperty("声部")
        private String subjectName;

        @ApiModelProperty("声部id")
        private Integer subjectId;

        @ApiModelProperty("课程级别")
        private Integer level;

        @ApiModelProperty(value = "学员适用范围ALL,RANGE,DISABLE")
        private String studentRangeType;

        @ApiModelProperty(value = "老师适用范围ALL,RANGE,DISABLE")
        private String teacherRangeType;

        @ApiModelProperty(value = "创建人")
        private Integer createBy;

        @ApiModelProperty(value = "修改人")
        private Integer updateBy;

        @ApiModelProperty(value = "创建人")
        private String createName;

        @ApiModelProperty(value = "修改人")
        private String updateName;

        @ApiModelProperty(value = "创建时间")
        private String createTime;


        @ApiModelProperty("使用状态 锁定： LOCK 解锁：UNLOCK")
        private String useStatus;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("CourseScheduleCoursewareDetail-课程计划课件详情")
    public static class CourseScheduleCoursewareDetail {

        // 课程计划编号
        @ApiModelProperty("课程计划编号")
        private Long lessonCoursewareId;

        // 课程计划编号
        @ApiModelProperty("课程计划详情编号")
        private Long lessonCoursewareDetailId;

        @ApiModelProperty("课程计划详情编号")
        private String coursewareDetailId;

        // 课件名称
        @ApiModelProperty("课件详情名称")
        private String coursewareDetailName;

        @ApiModelProperty("课时目标描述")
        private String lessonTargetDesc;

        @ApiModelProperty("知识点")
        private List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList;

        public void setLessonCoursewareDetailId(Long lessonCoursewareDetailId) {
        	this.lessonCoursewareDetailId = lessonCoursewareDetailId;
            this.coursewareDetailId = String.valueOf(lessonCoursewareDetailId);
        }

    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("updateRangeTypeDto-修改适用范围")
    public static class UpdateRange{

        @ApiModelProperty(value = "课件编号")
        private Integer lessonCoursewareId;

        @ApiModelProperty(value = "学员适用范围ALL,RANGE,DISABLE")
        private String studentRangeType;

        @ApiModelProperty(value = "学员编号")
        private List<Integer> studentIds;

        @ApiModelProperty(value = "老师适用范围ALL,RANGE,DISABLE")
        private String teacherRangeType;

        @ApiModelProperty(value = "老师编号")
        private List<Integer> teacherIds;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("UpdateRangeDetail-获取适用范围")
    public static class UpdateRangeDetail{

        @ApiModelProperty(value = "课件编号")
        private Integer lessonCoursewareId;

        @ApiModelProperty(value = "课件名称")
        private String name;

        @ApiModelProperty(value = "学员适用范围ALL,RANGE,DISABLE")
        private String studentRangeType;

        @ApiModelProperty(value = "老师适用范围ALL,RANGE,DISABLE")
        private String teacherRangeType;

        @ApiModelProperty(value = "适用学员列表")
        private List<SimpleUserDto> studentList;

        @ApiModelProperty(value = "适用老师列表")
        private List<SimpleUserDto> teacherList;
    }

    @Data
    public static class RefLevelDetailInfo{
        @ApiModelProperty("课件ID")
        private String id;

        @ApiModelProperty("教材ID")
        private Long lessonCoursewareId;
        @ApiModelProperty("教材名称")
        private String lessonCoursewareName;

        @ApiModelProperty("封面")
        private String coverImg;

        @ApiModelProperty("课程类型")
        private String courseTypeCode;

        @ApiModelProperty("是否锁定")
        private Boolean lockFlag;

        @ApiModelProperty("是否当前使用")
        private Boolean useFlag;

        @ApiModelProperty("使用状态 锁定： LOCK 解锁：UNLOCK")
        private String useStatus;
    }

    @Data
    public static class RefLevelQuery {

        @ApiModelProperty(value = "课件详情ID",required = true)
        private String lessonCoursewareDetailId;

        @ApiModelProperty(value = "老师ID",hidden = true)
        private Integer teacherId;

        private ClientEnum client;
    }

    @Data
    public static class LessonCoursewareExtend extends CbsLessonCoursewareWrapper.LessonCourseware{

        @ApiModelProperty("使用状态 锁定： LOCK 解锁：UNLOCK")
        private String useStatus;
    }

    @Data
    public static class LessonCoursewareDetailQuery{


        @ApiModelProperty(value = "课件详情ID")
        private Long lessonCoursewareDetailId;

        @ApiModelProperty("用户类型")
        private ClientEnum userType;

        @ApiModelProperty("机构可用曲目分类ID")
        private List<Integer> tenantCategoriesIdList;

        @ApiModelProperty("学生会员曲目分类ID")
        private List<Integer> studentCategoriesIdList;

    }

    @Data
    public static class AddMaterialRefExtend{

        @ApiModelProperty("使用状态 锁定： LOCK 解锁：UNLOCK")
        private String useStatus;
    }
}