package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayaedu.cbs.common.enums.school.EMaterialType;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsMaterialRefWrapper;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.CloudTeacherOrderDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.mapper.LessonCoursewareMapper;
import com.ym.mec.biz.dal.wrapper.LessonCoursewareWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class LessonCoursewareServiceImpl extends ServiceImpl<LessonCoursewareMapper, LessonCourseware> implements LessonCoursewareService {

    private final static Logger log = LoggerFactory.getLogger(LessonCoursewareServiceImpl.class);


    @Override
    public LessonCoursewareMapper getDao() {
        return this.baseMapper;
    }
    @Resource
    private CoursewareFeignService coursewareFeignService;
    @Resource
    private LessonCoursewareUserMapperService lessonCoursewareUserMapperService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SubjectDao subjectDao;
    @Resource
    private TeacherDao teacherDao;
    @Resource
    private SysMusicScoreService sysMusicScoreService;

    @Autowired
    private CloudTeacherOrderDao cloudTeacherOrderDao;

    @Autowired
    private MemberRankCourseTypeMapperService memberRankCourseTypeMapperService;

    @Override
    public IPage<LessonCoursewareWrapper.LessonCoursewareDto> selectPage(IPage<LessonCoursewareWrapper.LessonCoursewareDto> page, LessonCoursewareWrapper.LessonCoursewareQuery query) {
        if(StringUtils.isNotEmpty(query.getSearch())){
            List<LessonCoursewareWrapper.LessonCoursewareDto> dtos = baseMapper.list(query);
            if(CollectionUtils.isNotEmpty(dtos)){
                List<Long> lessonCoursewareIds = dtos.stream().map(LessonCoursewareWrapper.LessonCoursewareDto::getLessonCoursewareId).collect(Collectors.toList());
                CbsLessonCoursewareWrapper.LambdaQuery lambdaQuery = new CbsLessonCoursewareWrapper.LambdaQuery();
                lambdaQuery.setIds(lessonCoursewareIds);
                lambdaQuery.setLevel(query.getLevel());
                lambdaQuery.setName(query.getSearch());
                List<CbsLessonCoursewareWrapper.Entity> entityList = coursewareFeignService.lessonCoursewareLambdaQuery(lambdaQuery).feignData();
                if(CollectionUtils.isNotEmpty(entityList)){
                    List<Long> collect = entityList.stream().map(e -> e.getId()).collect(Collectors.toList());
                    query.setLessonCoursewareIds(collect);
                }else {
                    return page;
                }
            }
        }
        List<LessonCoursewareWrapper.LessonCoursewareDto> dtos = baseMapper.selectPage(page, query);
        if(CollectionUtils.isNotEmpty(dtos)){
            List<Long> lessonCoursewareIds = dtos.stream().map(LessonCoursewareWrapper.LessonCoursewareDto::getLessonCoursewareId).collect(Collectors.toList());
            CbsLessonCoursewareWrapper.LambdaQuery lambdaQuery = new CbsLessonCoursewareWrapper.LambdaQuery();
            lambdaQuery.setIds(lessonCoursewareIds);
            List<CbsLessonCoursewareWrapper.Entity> entityList = coursewareFeignService.lessonCoursewareLambdaQuery(lambdaQuery).feignData();
            if(CollectionUtils.isNotEmpty(entityList)){
                Map<Long, CbsLessonCoursewareWrapper.Entity> entityMap = entityList.stream().collect(Collectors.groupingBy(CbsLessonCoursewareWrapper.Entity::getId,
                        Collectors.collectingAndThen(Collectors.toList(), e -> e.get(0))));
                List<Integer> collect = dtos.stream().map(e -> e.getSubjectId()).collect(Collectors.toList());
                String subjectIds = collect.stream().map(e -> String.valueOf(e)).collect(Collectors.joining(","));
                List<Map<Integer, String>> subjectNames = subjectDao.queryNameByIds(subjectIds);
                Map<Integer, String> subjectNameMap = MapUtil.convertMybatisMap(subjectNames);

                List<Integer> userIds = dtos.stream().map(e -> e.getCreateBy()).filter(Objects::nonNull).distinct().collect(Collectors.toList());
                userIds.addAll(dtos.stream().map(e -> e.getUpdateBy()).filter(Objects::nonNull).distinct().collect(Collectors.toList()));
                userIds.removeAll(Collections.singleton(null));
                Map<Integer, String> userNameMap = new HashMap<>();
                if(CollectionUtils.isNotEmpty(userIds)){
                    userNameMap = MapUtil.convertMybatisMap(teacherDao.queryNameByIdList(userIds));
                }
                for (LessonCoursewareWrapper.LessonCoursewareDto e : dtos) {
                    CbsLessonCoursewareWrapper.Entity entity = entityMap.get(e.getLessonCoursewareId());
                    if (Objects.isNull(entity)) {
                        continue;
                    }
                    e.setCreateName(userNameMap.get(e.getCreateBy()));
                    e.setUpdateName(userNameMap.get(e.getUpdateBy()));
                    e.setSubjectName(subjectNameMap.get(e.getSubjectId()));
                    e.setName(entity.getName());
                    e.setCover(entity.getCoverImg());
                    e.setCourseNum(entity.getCourseNum());
                    e.setLevel(entity.getLevel());
                }
            }
        }
        return page.setRecords(dtos);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRangeType(LessonCoursewareWrapper.UpdateRange updateRange) {
        LessonCourseware lessonCourseware = this.getById(updateRange.getLessonCoursewareId());
        if (Objects.isNull(lessonCourseware)) {
            throw new RuntimeException("课件不存在");
        }
        lessonCourseware.setStudentRangeType(updateRange.getStudentRangeType());
        lessonCourseware.setTeacherRangeType(updateRange.getTeacherRangeType());
        lessonCourseware.setUpdateBy(sysUserService.getUserId());
        this.updateById(lessonCourseware);
        lessonCoursewareUserMapperService.lambdaUpdate()
                .eq(LessonCoursewareUserMapper::getLessonCoursewareId, updateRange.getLessonCoursewareId()).remove();
        if(StringUtils.equals(updateRange.getStudentRangeType(), "RANGE") && CollectionUtils.isNotEmpty(updateRange.getStudentIds())){
            List<LessonCoursewareUserMapper> list = updateRange.getStudentIds().stream().map(e -> {
                LessonCoursewareUserMapper mapper = new LessonCoursewareUserMapper();
                mapper.setLessonCoursewareId(updateRange.getLessonCoursewareId());
                mapper.setUserId(e);
                mapper.setUserType("STUDENT");
                return mapper;
            }).collect(Collectors.toList());
            lessonCoursewareUserMapperService.saveBatch(list);
        }
        if(StringUtils.equals(updateRange.getTeacherRangeType(), "RANGE") && CollectionUtils.isNotEmpty(updateRange.getTeacherIds())){
            List<LessonCoursewareUserMapper> list = updateRange.getTeacherIds().stream().map(e -> {
                LessonCoursewareUserMapper mapper = new LessonCoursewareUserMapper();
                mapper.setLessonCoursewareId(updateRange.getLessonCoursewareId());
                mapper.setUserId(e);
                mapper.setUserType("TEACHER");
                return mapper;
            }).collect(Collectors.toList());
            lessonCoursewareUserMapperService.saveBatch(list);
        }
    }

    @Override
    public LessonCoursewareWrapper.UpdateRangeDetail getRangeType(Integer lessonCoursewareId) {
        LessonCourseware lessonCourseware = this.getById(lessonCoursewareId);
        if (Objects.isNull(lessonCourseware)) {
            throw new RuntimeException("课件不存在");
        }
        R<CbsLessonCoursewareWrapper.LessonCourseware> lessonCoursewareR = coursewareFeignService.lessonCoursewareDetail(lessonCourseware.getLessonCourseId());
        if (!lessonCoursewareR.getCode().equals(200)) {
            throw new BizException("获取课件详情失败 {}", lessonCoursewareR.getMessage());
        }
        LessonCoursewareWrapper.UpdateRangeDetail detail = new LessonCoursewareWrapper.UpdateRangeDetail();
        detail.setName(lessonCoursewareR.getData().getName());
        detail.setLessonCoursewareId(lessonCoursewareId);
        detail.setStudentRangeType(lessonCourseware.getStudentRangeType());
        detail.setTeacherRangeType(lessonCourseware.getTeacherRangeType());
        if(lessonCourseware.getStudentRangeType().equals("RANGE")){
            List<LessonCoursewareUserMapper> studentList = lessonCoursewareUserMapperService.lambdaQuery()
                    .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCoursewareId)
                    .eq(LessonCoursewareUserMapper::getUserType, "STUDENT").list();
            if(CollectionUtils.isNotEmpty(studentList)){
                List<Integer> studentIds = studentList.stream().map(LessonCoursewareUserMapper::getUserId).collect(Collectors.toList());
                List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(studentIds);
                if (CollectionUtils.isNotEmpty(userDtoList)) {
                    for (SimpleUserDto userDto : userDtoList) {
                        userDto.setUserName(userDto.getNickName());
                    }
                }
                detail.setStudentList(userDtoList);
            }
        }
        if(lessonCourseware.getTeacherRangeType().equals("RANGE")){
            List<LessonCoursewareUserMapper> teacherList = lessonCoursewareUserMapperService.lambdaQuery()
                    .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCoursewareId)
                    .eq(LessonCoursewareUserMapper::getUserType, "TEACHER").list();
            if(CollectionUtils.isNotEmpty(teacherList)){
                List<Integer> teacherIds = teacherList.stream().map(LessonCoursewareUserMapper::getUserId).collect(Collectors.toList());
                List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(teacherIds);
                detail.setTeacherList(userDtoList);
            }
        }
        return detail;
    }

    @Override
    public CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail getLessonCourseDetail(LessonCoursewareWrapper.LessonCoursewareDetailQuery query) {
        CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail detail = coursewareFeignService
            .lessonCoursewareDetailDetail(query.getLessonCoursewareDetailId()).feignData();
        if (detail == null) {
            return null;
        }
        //获取课件下面包含的所有的曲目编号
        List<Long> musicScoreIds = new ArrayList<>();
        this.setMusicSheetId(detail.getKnowledgePointList(),musicScoreIds);
        Map<Long, SysMusicScore> musicScoreMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(musicScoreIds)) {
            List<SysMusicScore> musicScores = sysMusicScoreService.getDao().findByCbsId(musicScoreIds,false);
            musicScoreMap = musicScores.stream().collect(Collectors.toMap(e -> Long.parseLong(e.getCbsMusicSheetId()), o -> o));
        }
        //获取知识点下的素材列表、下面关联的资源列表,如果applicationIdList是空的，那么把他剔除掉
        this.setTreeResourceId(detail.getKnowledgePointList(),query,musicScoreMap);
        return detail;
    }

    @Override
    public List<LessonCoursewareWrapper.RefLevelDetailInfo> refLevel(LessonCoursewareWrapper.RefLevelQuery query) {
        List<LessonCoursewareWrapper.RefLevelDetailInfo> result = new ArrayList<>();
        CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail lessonCoursewareDetail =
                coursewareFeignService.lessonCoursewareDetailDetail(Long.parseLong(query.getLessonCoursewareDetailId())).feignData();
        if (lessonCoursewareDetail == null) {
            throw new BizException("课件未找到");
        }
        CbsLessonCoursewareWrapper.LessonCourseware lessonCourseware =
                coursewareFeignService.lessonCoursewareDetail(lessonCoursewareDetail.getLessonCoursewareId()).feignData();
        if (lessonCourseware == null) {
            throw new BizException("课件未找到");
        }
        if (query.getClient() == ClientEnum.TEACHER) {
            Teacher teacher = teacherDao.get(query.getTeacherId());
            if (teacher == null) {
                throw new BizException("老师信息不存在");
            }
            if (lessonCoursewareDetail.getLevel() == null || teacher.getCoursewareFlag()) {
                result.add(this.initRefLevelDetailInfo(lessonCoursewareDetail, lessonCourseware));
                return result;
            }
        } else {
            if (lessonCoursewareDetail.getLevel() == null) {
                result.add(this.initRefLevelDetailInfo(lessonCoursewareDetail, lessonCourseware));
                return result;
            }
        }
        LessonCoursewareWrapper.LessonCoursewareQuery query1 = new LessonCoursewareWrapper.LessonCoursewareQuery();
        query1.setPage(1);
        query1.setRows(9999);
        if (query.getClient() == ClientEnum.STUDENT) {
            query1.setStudentId(query.getTeacherId());
        } else {
            query1.setTeacherId(query.getTeacherId());
        }
        query1.setEnable(true);
        List<LessonCoursewareWrapper.LessonCoursewareDto> dtos = baseMapper.list(query1);
        if(CollectionUtils.isEmpty(dtos)){
            result.add(this.initRefLevelDetailInfo(lessonCoursewareDetail,lessonCourseware));
            return result;
        }
        List<Long> lessonCoursewareIds = dtos.stream().map(LessonCoursewareWrapper.LessonCoursewareDto::getLessonCoursewareId).collect(Collectors.toList());
        // 查询当前能使用的课件
        CbsLessonCoursewareWrapper.LessonCoursewareQuery lambdaQuery = new CbsLessonCoursewareWrapper.LessonCoursewareQuery();
        lambdaQuery.setPage(1);
        lambdaQuery.setRows(9999);
        lambdaQuery.setLevel(lessonCourseware.getLevel());
        lambdaQuery.setLessonCoursewareIds(lessonCoursewareIds);
        List<CbsLessonCoursewareWrapper.LessonCourseware> lessonCoursewareList = coursewareFeignService.lessonCoursewarePage(lambdaQuery).feignData().getRows();
        if (CollectionUtils.isEmpty(lessonCoursewareList)) {
            result.add(this.initRefLevelDetailInfo(lessonCoursewareDetail,lessonCourseware));
            return result;
        }
        List<Long> courseIds = lessonCoursewareList.stream().map(CbsLessonCoursewareWrapper.LessonCourseware::getId).collect(Collectors.toList());
        Map<Long, CbsLessonCoursewareWrapper.LessonCourseware> lessonCoursewareMap =
                lessonCoursewareList.stream().collect(Collectors.toMap(CbsLessonCoursewareWrapper.LessonCourseware::getId, o -> o));

        // 查询可用的课件详情信息
        CbsLessonCoursewareDetailWrapper.LessonCoursewareDetailQuery lessonCoursewareDetailQuery = new CbsLessonCoursewareDetailWrapper.LessonCoursewareDetailQuery();
        lessonCoursewareDetailQuery.setPage(1);
        lessonCoursewareDetailQuery.setRows(9999);
        lessonCoursewareDetailQuery.setLessonCoursewareIdList(courseIds);
        lessonCoursewareDetailQuery.setLevel(lessonCoursewareDetail.getLevel());
        List<CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail> lessonCoursewareDetails = coursewareFeignService.lessonCoursewareDetailPage(lessonCoursewareDetailQuery).feignData().getRows();
        if (CollectionUtils.isEmpty(lessonCoursewareDetails)) {
            result.add(this.initRefLevelDetailInfo(lessonCoursewareDetail,lessonCourseware));
            return result;
        }
        List<Long> lessonIds = new ArrayList<>();
        if (query.getClient() == ClientEnum.STUDENT) {
            List<Integer> activationVipIds = cloudTeacherOrderDao.getActivationVipIds(query.getTeacherId());

            // 获取会员的曲目分类

            List<MemberRankCourseTypeMapper> courseTypeMapperList = memberRankCourseTypeMapperService.getByMemberRankId(activationVipIds);
            if (CollectionUtils.isEmpty(courseTypeMapperList)) {
                courseTypeMapperList = Lists.newArrayList();
            }
            List<Integer> categoryIds = courseTypeMapperList.stream().map(MemberRankCourseTypeMapper::getCourseType)
                .distinct().map(Integer::parseInt).collect(Collectors.toList());
            // 本地的课件ID 换远程ID
            lessonIds = getByLessonCoursewareIds(categoryIds).stream().map(LessonCourseware::getLessonCourseId).distinct().collect(Collectors.toList());
        }
        for (CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail coursewareDetail : lessonCoursewareDetails) {
            LessonCoursewareWrapper.RefLevelDetailInfo refLevelDetailInfo = new LessonCoursewareWrapper.RefLevelDetailInfo();
            refLevelDetailInfo.setId(coursewareDetail.getId());
            refLevelDetailInfo.setLessonCoursewareId(coursewareDetail.getLessonCoursewareId());
            CbsLessonCoursewareWrapper.LessonCourseware lessonCourseware1 = lessonCoursewareMap.get(coursewareDetail.getLessonCoursewareId());
            if (lessonCourseware1 == null) {
                continue;
            }
            refLevelDetailInfo.setCoverImg(lessonCourseware1.getCoverImg());
            refLevelDetailInfo.setCourseTypeCode(lessonCourseware1.getCourseTypeCode());
            refLevelDetailInfo.setLockFlag(false);
            refLevelDetailInfo.setLessonCoursewareName(lessonCourseware1.getName());
            refLevelDetailInfo.setUseFlag(coursewareDetail.getId().equals(query.getLessonCoursewareDetailId()));
            if (lessonIds.contains(lessonCourseware1.getId()) || query.getClient() == ClientEnum.TEACHER) {
                refLevelDetailInfo.setUseStatus("UNLOCK");
            } else refLevelDetailInfo.setUseStatus("LOCK");
            result.add(refLevelDetailInfo);
        }
        return result;
    }

    @Override
    public LessonCourseware getByLessonCoursewareId(Long lessonCoursewareId) {
        List<LessonCourseware> list = lambdaQuery()
            .eq(LessonCourseware::getLessonCourseId, lessonCoursewareId)
            .list();
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public List<LessonCourseware> getByLessonCoursewareIds(List<Integer> categoryIds) {
        if (CollectionUtils.isEmpty(categoryIds)) {
            return new ArrayList<>();
        }
        return this.lambdaQuery()
            .in(LessonCourseware::getId, categoryIds)
            .list();
    }

    private LessonCoursewareWrapper.RefLevelDetailInfo initRefLevelDetailInfo(CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail coursewareDetail,
                                                                              CbsLessonCoursewareWrapper.LessonCourseware lessonCourseware) {
        LessonCoursewareWrapper.RefLevelDetailInfo refLevelDetailInfo = new LessonCoursewareWrapper.RefLevelDetailInfo();
        refLevelDetailInfo.setId(coursewareDetail.getId());
        refLevelDetailInfo.setLessonCoursewareId(coursewareDetail.getLessonCoursewareId());
        refLevelDetailInfo.setCoverImg(lessonCourseware.getCoverImg());
        refLevelDetailInfo.setCourseTypeCode(lessonCourseware.getCourseTypeCode());
        refLevelDetailInfo.setLockFlag(false);
        refLevelDetailInfo.setLessonCoursewareName(lessonCourseware.getName());
        refLevelDetailInfo.setUseFlag(true);
        return refLevelDetailInfo;
    }

    //递归处理CbsLessonCoursewareDetailWrapper.KnowledgePointSmall
    @Override
    public void setTreeResourceId(List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList,
                                  LessonCoursewareWrapper.LessonCoursewareDetailQuery query,
                                  Map<Long, SysMusicScore> musicScores){
        if(CollectionUtils.isNotEmpty(knowledgePointList)){
            knowledgePointList.forEach(knowledgePointSmall -> {
                this.setResourceId(knowledgePointSmall,query,musicScores);
                List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> children = knowledgePointSmall.getChildren();
                if(CollectionUtils.isNotEmpty(children)){
                    this.setTreeResourceId(children,query,musicScores);
                }
            });
        }
    }

    @Override
    public List<Long> getMusicScoreIds(List<CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetails) {
        List<Long> musicScoreIds = new ArrayList<>();
        //获取所有曲目的ID
        if(CollectionUtils.isNotEmpty(courseScheduleCoursewareDetails)){
            courseScheduleCoursewareDetails.forEach(courseScheduleCoursewareDetail -> {
                this.setMusicSheetId(courseScheduleCoursewareDetail.getKnowledgePointList(),musicScoreIds);
            });
        }
        return musicScoreIds;
    }

    private void setMusicSheetId(List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList,List<Long> musicScoreIds){
        if(CollectionUtils.isNotEmpty(knowledgePointList)){
            for (CbsLessonCoursewareDetailWrapper.KnowledgePointSmall knowledgePointSmall : knowledgePointList) {
                List<CbsLessonCoursewareDetailWrapper.MaterialSmall> materialList = knowledgePointSmall.getMaterialList();
                if(CollectionUtils.isNotEmpty(materialList)){
                    //获取所有曲目的ID
                    List<Long> musicScoreIds1 = materialList.stream().filter(materialSmall -> materialSmall.getType() == EMaterialType.SONG)
                            .map(materialSmall -> Long.parseLong(materialSmall.getContent())).collect(Collectors.toList());
                    musicScoreIds.addAll(musicScoreIds1);
                    for (CbsLessonCoursewareDetailWrapper.MaterialSmall materialSmall : materialList) {
                        List<CbsMaterialRefWrapper.AddMaterialRef> materialRefs = materialSmall.getMaterialRefs();
                        if(CollectionUtils.isNotEmpty(materialRefs)){
                            //获取所有曲目的ID
                            List<Long> musicScoreIds2 = materialRefs.stream().filter(materialRef -> materialRef.getRelateMaterialInfo() != null
                                    && StringUtils.equals(materialRef.getRelateMaterialInfo().getType(),"SONG")
                                    && CollectionUtils.isNotEmpty(materialRef.getRelateMaterialInfo().getApplicationIdList()))
                                    .map(materialRef -> materialRef.getRelateMaterialInfo().getId()).collect(Collectors.toList());
                            musicScoreIds.addAll(musicScoreIds2);
                        }
                    }
                }
                List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> children = knowledgePointSmall.getChildren();
                if(CollectionUtils.isNotEmpty(children)){
                    this.setMusicSheetId(children,musicScoreIds);
                }
            }
        }
    }


    private void setResourceId(CbsLessonCoursewareDetailWrapper.KnowledgePointSmall knowledgePointSmall,
                               LessonCoursewareWrapper.LessonCoursewareDetailQuery query,
                               Map<Long, SysMusicScore> musicScoreMap){
        List<CbsLessonCoursewareDetailWrapper.MaterialSmall> materialList = knowledgePointSmall.getMaterialList();
        if(CollectionUtils.isNotEmpty(materialList)){
            for (CbsLessonCoursewareDetailWrapper.MaterialSmall materialSmall : materialList) {
                if(materialSmall.getType() == EMaterialType.SONG){
                    SysMusicScore musicScore = musicScoreMap.get(Long.parseLong(materialSmall.getContent()));
                    //没有启用的曲目不显示
                    if(musicScore != null && musicScore.getShowFlag() == 1 && musicScore.getClientShowFlag() == 1) {
                        materialSmall.setContent(musicScore.getId().toString());
                    }else {
                        materialSmall.setContent(null);
                    }
                }
                //
                List<CbsMaterialRefWrapper.AddMaterialRef> materialRefs = materialSmall.getMaterialRefs();
                if (CollectionUtils.isNotEmpty(materialRefs)){
                    for (CbsMaterialRefWrapper.AddMaterialRef materialRef : materialRefs) {
                        CbsMaterialRefWrapper.RelateMaterialInfo relateMaterialInfo = materialRef.getRelateMaterialInfo();
                        if(relateMaterialInfo != null && StringUtils.equals(relateMaterialInfo.getType(),"SONG")
                                && CollectionUtils.isNotEmpty(relateMaterialInfo.getApplicationIdList())){
                            SysMusicScore musicScore = musicScoreMap.get(relateMaterialInfo.getId());
                            if(musicScore != null && musicScore.getShowFlag() == 1 && musicScore.getClientShowFlag() == 1
                                    && query.getTenantCategoriesIdList().contains(musicScore.getCbsMusicCategoriesId())) {
                                materialRef.setResourceIdStr(musicScore.getId().toString());
                                // 学生需要判断学练宝是否能用
                                LessonCoursewareWrapper.AddMaterialRefExtend addMaterialRefExtend = new LessonCoursewareWrapper.AddMaterialRefExtend();
                                if (query.getUserType() !=null && query.getUserType() == ClientEnum.STUDENT) {
                                    if (StringUtils.isBlank(musicScore.getRankIds())) {
                                        addMaterialRefExtend.setUseStatus("UNLOCK");
                                    } else {
                                        addMaterialRefExtend.setUseStatus(query.getStudentCategoriesIdList()
                                            .contains(musicScore.getCbsMusicCategoriesId()) ? "UNLOCK" : "LOCK");
                                    }
                                } else {
                                    addMaterialRefExtend.setUseStatus("UNLOCK");
                                }
                                materialRef.setExtend(addMaterialRefExtend);
                            }
                        }
                    }
                }
            }
            //剔除掉视频素材，且没有content的数据,且不在可见分类内的数据
            materialList.removeIf(materialSmall -> materialSmall.getType() == EMaterialType.SONG
                    && StringUtils.isEmpty(materialSmall.getContent()));
        }
    }
}

