package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MemberPrivilegesItemDao;
import com.ym.mec.biz.dal.dao.MemberRankPrivilegesDao;
import com.ym.mec.biz.dal.entity.MemberPrivilegesItem;
import com.ym.mec.biz.dal.entity.MemberRankPrivileges;
import com.ym.mec.biz.service.MemberRankPrivilegesService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class MemberRankPrivilegesServiceImpl extends BaseServiceImpl<Integer, MemberRankPrivileges> implements MemberRankPrivilegesService {

	@Resource
	private MemberRankPrivilegesDao memberRankPrivilegesDao;

	@Resource
	private MemberPrivilegesItemDao memberPrivilegesItemDao;

	@Override
	public BaseDAO<Integer, MemberRankPrivileges> getDAO() {
		return memberRankPrivilegesDao;
	}

	@Override
	public List<MemberPrivilegesItem> queryByMemberRankId(Integer memberRankId) {
		List<Integer> memberPrivilegesIdList = memberRankPrivilegesDao.
				findByRankSettingId(memberRankId).stream().map(MemberRankPrivileges::getMemberPrivilegesId)
				.collect(Collectors.toList());
		
		if (CollectionUtils.isEmpty(memberPrivilegesIdList)) {
			return new ArrayList<>();
		}
		return initMemberPrivilegesItem(memberPrivilegesIdList);
	}

	@Override
	public List<MemberPrivilegesItem> queryByMemberRankIds(List<Integer> memberRankIds) {
		List<Integer> memberPrivilegesIdList = memberRankPrivilegesDao.
				findByRankSettingIds(memberRankIds).stream().map(MemberRankPrivileges::getMemberPrivilegesId)
				.collect(Collectors.toList());

		if (CollectionUtils.isEmpty(memberPrivilegesIdList)) {
			return new ArrayList<>();
		}
		return initMemberPrivilegesItem(memberPrivilegesIdList);
	}

	private List<MemberPrivilegesItem> initMemberPrivilegesItem(List<Integer> memberPrivilegesIdList) {
		List<MemberPrivilegesItem> memberPrivilegesItemList = memberPrivilegesItemDao.findById(memberPrivilegesIdList);

		Map<Integer, MemberPrivilegesItem> map = new HashMap<Integer, MemberPrivilegesItem>();

		for (MemberPrivilegesItem item : memberPrivilegesItemList) {
			if (!map.containsKey(item.getId())) {
				MemberPrivilegesItem memberPrivilegesItem = queryParentBysubId(item);

				if (map.containsKey(memberPrivilegesItem.getId())) {
					memberPrivilegesItem = addSubItemToParent(map.get(memberPrivilegesItem.getId()), item);
				}
				map.put(memberPrivilegesItem.getId(), memberPrivilegesItem);
			}
		}

		return new ArrayList<>(map.values());
	}

	private MemberPrivilegesItem queryParentBysubId(MemberPrivilegesItem memberPrivilegesItem) {

		if (memberPrivilegesItem != null && (memberPrivilegesItem.getParentId() != 0 && memberPrivilegesItem.getParentId() != null)) {
			Integer parentId = memberPrivilegesItem.getParentId();
			MemberPrivilegesItem parentItem = memberPrivilegesItemDao.get(parentId);
			if (parentItem != null) {
				parentItem.getMemberPrivilegesItems().add(memberPrivilegesItem);
				return queryParentBysubId(parentItem);
			}
		}

		return memberPrivilegesItem;
	}

	private MemberPrivilegesItem addSubItemToParent(MemberPrivilegesItem parentItem, MemberPrivilegesItem subItem) {
		if (subItem.getParentId().equals(parentItem.getId())) {
			parentItem.getMemberPrivilegesItems().add(subItem);
		} else {
			for (MemberPrivilegesItem item : parentItem.getMemberPrivilegesItems()) {
				addSubItemToParent(item, subItem);
			}
		}

		return parentItem;
	}

}