package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ym.mec.biz.dal.dao.CloudTeacherOrderDao;
import com.ym.mec.biz.dal.dao.StudentLessonExaminationDetailDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.LessonExamination;
import com.ym.mec.biz.dal.entity.MemberRankCategoryMapper;
import com.ym.mec.biz.dal.entity.StudentLessonExaminationDetail;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryDto;
import com.ym.mec.biz.service.LessonExaminationService;
import com.ym.mec.biz.service.MemberRankCategoryMapperService;
import com.ym.mec.biz.service.StudentLessonExaminationDetailService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 进度评测详情表(StudentLessonExaminationDetail)表服务实现类
 *
 * @author zx
 * @since 2023-04-03 18:22:01
 */
@Service("studentLessonExaminationDetailService")
public class StudentLessonExaminationDetailServiceImpl extends ServiceImpl<StudentLessonExaminationDetailDao, StudentLessonExaminationDetail> implements StudentLessonExaminationDetailService {

    private final static Logger log = LoggerFactory.getLogger(StudentLessonExaminationDetailServiceImpl.class);

    @Autowired
    private LessonExaminationService lessonExaminationService;

    @Autowired
    private SysMusicScoreDao sysMusicScoreDao;

    @Autowired
    private CloudTeacherOrderDao cloudTeacherOrderDao;

    @Autowired
    private MemberRankCategoryMapperService memberRankCategoryMapperService;

    @Override
    public StudentLessonExaminationDetailDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(Long lessonExaminationId,List<StudentLessonExaminationSaveDto> studentLessonExaminationSaveDtos) {
        List<StudentLessonExaminationDetail> detailList = new ArrayList<>();
        for (StudentLessonExaminationSaveDto dto : studentLessonExaminationSaveDtos) {
            String[] studentIds = dto.getStudentIds().split(",");
            List<LessonExaminationDetailDto> detailDtos = dto.getExaminationDetailDtos();
            if (CollectionUtils.isEmpty(detailDtos)) {
                throw new BizException("有部分声部未设置作业内容");
            }
            for (String id : studentIds) {
                Integer studentId = Integer.parseInt(id);
                Integer score = 100 / detailDtos.size();
                Integer subScore = 100 - score * detailDtos.size();
                for (int i = 0; i < detailDtos.size(); i++) {
                    LessonExaminationDetailDto vo = detailDtos.get(i);
                    StudentLessonExaminationDetail studentLessonExaminationDetail = new StudentLessonExaminationDetail();
                    studentLessonExaminationDetail.setUserId(studentId);
                    studentLessonExaminationDetail.setMemo(dto.getMemo());
                    studentLessonExaminationDetail.setLessonExaminationId(lessonExaminationId);
                    studentLessonExaminationDetail.setMusicScoreId(vo.getMusicScoreId());
                    studentLessonExaminationDetail.setHeardLevel(vo.getHeardLevel());
                    studentLessonExaminationDetail.setStandardScore(vo.getScore());
                    studentLessonExaminationDetail.setStart(vo.getStart());
                    studentLessonExaminationDetail.setHomeworkType(vo.getHomeworkType());
                    studentLessonExaminationDetail.setDesc(vo.getDesc());
                    studentLessonExaminationDetail.setEnd(vo.getEnd());
                    studentLessonExaminationDetail.setGroup(dto.getGroup());
                    if (i == 0) {
                        studentLessonExaminationDetail.setAvgScore(score + subScore);
                    } else {
                        studentLessonExaminationDetail.setAvgScore(score);
                    }
                    studentLessonExaminationDetail.setPartIndex(vo.getPartIndex());
                    detailList.add(studentLessonExaminationDetail);
                }
            }
        }
        baseMapper.insertBatch(detailList);
    }

    @Override
    public List<StudentLessonExaminationDetailDto> queryAll(StudentLessonExaminationQueryDto query) {

        List<StudentLessonExaminationDetailDto> studentLessonExaminationDetailDtos = baseMapper.queryAll(query);
        if (CollectionUtils.isEmpty(studentLessonExaminationDetailDtos)) {
            return new ArrayList<>();
        }
        for (StudentLessonExaminationDetailDto studentLessonExaminationDetailDto : studentLessonExaminationDetailDtos) {
            if ("DESCRIPTION".equals(studentLessonExaminationDetailDto.getHomeworkType())){
                studentLessonExaminationDetailDto.setUseStatus("UNLOCK");
            }
            if (StringUtils.isEmpty(studentLessonExaminationDetailDto.getFileJsons())) {
                continue;
            }
            studentLessonExaminationDetailDto.setFileJson(JSON.parseArray(studentLessonExaminationDetailDto.getFileJsons(), CourseHomeworkWrapper.FileJson.class));

            studentLessonExaminationDetailDto.setFileJsons("");
        }
        // 曲目id集合
        List<Integer> musicScoreIdList = studentLessonExaminationDetailDtos.stream()
            .filter(o-> o.getHomeworkType().equals("MUSIC_SCORE"))
            .map(StudentLessonExaminationDetailDto::getMusicScoreId)
            .distinct()
            .collect(Collectors.toList());

        if(org.apache.commons.collections.CollectionUtils.isNotEmpty(musicScoreIdList)) {

            // 查询有效的会员
            List<Integer> activationVipIds = cloudTeacherOrderDao.getActivationVipIds(query.getUserId());

            // 获取会员的曲目分类
            List<MemberRankCategoryMapper> categoryMapperList = memberRankCategoryMapperService.getByMemberRankId(activationVipIds);
            if (org.apache.commons.collections.CollectionUtils.isEmpty(categoryMapperList)) {
                categoryMapperList = Lists.newArrayList();
            }
            List<Integer> categoryIds = categoryMapperList.stream().map(MemberRankCategoryMapper::getCategoryId).distinct().collect(Collectors.toList());

            // 查询曲目信息
            List<SysMusicScore> scoreList = sysMusicScoreDao.findByIds(musicScoreIdList.stream().map(String::valueOf).collect(Collectors.joining(",")));
            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(scoreList)) {
                Map<Integer, SysMusicScore> musicScoreMap = scoreList.stream().collect(Collectors.toMap(SysMusicScore::getId, Function.identity()));
                for (StudentLessonExaminationDetailDto studentLessonExaminationDetailDto : studentLessonExaminationDetailDtos) {
                    if (org.apache.commons.lang3.StringUtils.equals(studentLessonExaminationDetailDto.getHomeworkType(), "MUSIC_SCORE")) {
                        SysMusicScore sysMusicScore = musicScoreMap.get(studentLessonExaminationDetailDto.getMusicScoreId());
                        if (sysMusicScore != null) {

                            // 用户VIP可查看曲目分类
                            // 设置默认不可以查看
                            if (categoryIds.contains(sysMusicScore.getCbsMusicCategoriesId())
                                || org.apache.commons.lang3.StringUtils.isBlank(sysMusicScore.getRankIds())) {
                                // 免费曲目，已购买曲目可直接查看
                                studentLessonExaminationDetailDto.setUseStatus("UNLOCK");
                            } else {
                                studentLessonExaminationDetailDto.setUseStatus("LOCK");
                            }
                        }
                    } else if ("DESCRIPTION".equals(studentLessonExaminationDetailDto.getHomeworkType())){
                        studentLessonExaminationDetailDto.setUseStatus("UNLOCK");
                    }
                }
            }
        }
        if (query.getLessonExaminationId() != null) {
            LessonExamination lessonExamination = lessonExaminationService.getById(query.getLessonExaminationId());
            if (lessonExamination != null) {
                boolean expireFlag = false;
                if (lessonExamination.getFileExpireDay() != null && lessonExamination.getFileExpireDay() >0) {
                    expireFlag = DateUtil.addDays(lessonExamination.getExpireDate(), lessonExamination.getFileExpireDay()).before(new Date());
                }
                for (StudentLessonExaminationDetailDto dto : studentLessonExaminationDetailDtos) {
                    if (dto.getFileJson() != null) {
                        for (CourseHomeworkWrapper.FileJson fileJson : dto.getFileJson()) {
                            fileJson.setExpireFlag(expireFlag);
                        };
                    }
                }
            }
        }
        return studentLessonExaminationDetailDtos;
    }

    @Override
    public List<StudentLessonExaminationDetail> queryByStudent(Integer userId, Long lessonExaminationId) {
        return this.lambdaQuery()
            .eq(StudentLessonExaminationDetail::getUserId, userId)
            .eq(StudentLessonExaminationDetail::getLessonExaminationId, lessonExaminationId)
            .list();
    }

}

